@@ -14,6 +14,18 @@ import pandas as pd
 # local
 
 from . config import setup_config
+from . constants import (
+    FILE_FIELD,
+    INPUT_FIELD,
+    STAGING_FIELD,
+)
+from . functions.assign_id import (
+    assign_id,
+    create_id_context_map,
+    setup_assign_ids,
+)
+from . functions.get_field_value import get_field_value
+from . functions.set_field_value import set_field_value
 
 dict_loaders: dict[str, callable] = {}
 def register_loader(
@@ -40,6 +52,15 @@ def load_excel(
     df = pd.read_excel(input_file)
     return df
 
+@register_loader('.json')
+def load_json(
+    input_file: str,
+):
+    with open(input_file, 'r') as f:
+        data = json.load(f)
+    df = pd.DataFrame(data)
+    return df
+
 @register_saver('.json')
 def save_json(
     df: pd.DataFrame,
@@ -85,41 +106,16 @@ def save_jsonl(
             )
             f.write('\n')
 
-def set_field_value(
-    data: OrderedDict,
-    field: str,
-    value: any,
-):
-    if '.' in field:
-        field, rest = field.split('.', 1)
-        if field not in data:
-            data[field] = OrderedDict()
-        set_field_value(data[field], rest, value)
-    else:
-        data[field] = value
-
-def get_field_value(
-    data: OrderedDict,
-    field: str,
-):
-    if field in data:
-        return data[field], True
-    if '.' in field:
-        field, rest = field.split('.', 1)
-        if field in data:
-            return get_field_value(data[field], rest)
-    return None, False
-
 def search_column_value(
     row: OrderedDict,
     column: str,
 ):
-    if '__debug__' in row:
-        value, found = get_field_value(row['__debug__'], column)
+    if STAGING_FIELD in row:
+        value, found = get_field_value(row[STAGING_FIELD], column)
         if found:
             return value, True
-    value, found = get_field_value(row['__debug__'], column)
-    original, found = get_field_value(row, '__debug__.__original__')
+    value, found = get_field_value(row[STAGING_FIELD], column)
+    original, found = get_field_value(row, f'{STAGING_FIELD}.{INPUT_FIELD}')
     if found:
         value, found = get_field_value(original, column)
         if found:
@@ -137,7 +133,7 @@ def map_constants(
     new_row = OrderedDict(row)
     for column in dict_constants.keys():
         #set_field_value(new_row, column, dict_constants[column])
-        set_field_value(new_row, f'__debug__.{column}', dict_constants[column])
+        set_field_value(new_row, f'{STAGING_FIELD}.{column}', dict_constants[column])
     return new_row
 
 def map_formats(
@@ -147,16 +143,22 @@ def map_formats(
     new_row = OrderedDict(row)
     for column in dict_formats.keys():
         template = dict_formats[column]
-        try:
-            params = {}
-            params.update(row['__debug__'])
-            params.update(row)
-            formatted = template.format(**params)
-        except KeyError as e:
-            ic(e)
-            formatted = template
-            raise e
-        set_field_value(new_row, f'__debug__.{column}', formatted)
+        params = {}
+        params.update(row[STAGING_FIELD])
+        params.update(row)
+        formatted = None
+        while formatted is None:
+            try:
+                formatted = template.format(**params)
+            except KeyError as e:
+                #ic(e)
+                #ic(e.args)
+                #ic(e.args[0])
+                key = e.args[0]
+                params[key] = f'__{key}__undefined__'
+            except:
+                raise
+        set_field_value(new_row, f'{STAGING_FIELD}.{column}', formatted)
     return new_row
 
 def remap_columns(
@@ -169,7 +171,7 @@ def remap_columns(
         if found:
             set_field_value(new_row, column, value)
     for column in row.keys():
-        if column == '__debug__':
+        if column == STAGING_FIELD:
             # NOTE: Ignore debug fields
             set_field_value(new_row, column, row[column])
     return new_row
@@ -185,49 +187,9 @@ def apply_fields_split_by_newline(
         if found:
             if isinstance(value, str):
                 new_value = value.split('\n')
-                set_field_value(new_row, f'__debug__.{column}', new_value)
+                set_field_value(new_row, f'{STAGING_FIELD}.{column}', new_value)
             else:
-                set_field_value(new_row, f'__debug__.{column}', value)
-    return new_row
-
-def create_id_stat_node():
-    return {
-        'max_id': 0,
-        'dict_value_to_id': {},
-        'dict_id_to_node': {},
-    }
-
-def assign_id_in_node(
-    row: OrderedDict,
-    column: str,
-    dict_assignment: OrderedDict,
-    id_stat_node: dict,
-):
-    value, found = search_column_value(row, dict_assignment[column])
-    if not found:
-        raise KeyError(f'Column not found: {column}, existing columns: {row.keys()}')
-    if value not in id_stat_node['dict_value_to_id']:
-        field_id = id_stat_node['max_id'] + 1
-        id_stat_node['max_id'] = field_id
-        id_stat_node['dict_value_to_id'][value] = field_id
-        node = create_id_stat_node()
-        id_stat_node['dict_id_to_node'][field_id] = node
-    else:
-        field_id = id_stat_node['dict_value_to_id'][value]
-        node = id_stat_node['dict_id_to_node'][field_id]
-    set_field_value(row, f'__debug__.{column}', field_id)
-    set_field_value(row, f'__debug__.__ids__.{column}', field_id)
-    return node
-
-def assign_id(
-    row: OrderedDict,
-    dict_assignment: OrderedDict,
-    root_id_stat_node: dict,
-):
-    new_row = OrderedDict(row)
-    node = root_id_stat_node
-    for column in dict_assignment:
-        node = assign_id_in_node(new_row, column, dict_assignment, node)
+                set_field_value(new_row, f'{STAGING_FIELD}.{column}', value)
     return new_row
 
 def convert(
@@ -245,28 +207,23 @@ def convert(
     ic()
     ic(input_files)
     df_list = []
-    dict_constants: OrderedDict | None = None
-    dict_formats: OrderedDict | None = None
-    dict_assign_ids= None
-    root_id_stat = create_id_stat_node()
+    id_context_map = create_id_context_map()
     config = setup_config(config_path)
     ic(config)
     if assign_constants:
-        dict_constants = OrderedDict()
         fields = assign_constants.split(',')
         for field in fields:
             if '=' in field:
                 dst, src = field.split('=')
-                dict_constants[dst] = src
+                config.process.assign_constants[dst] = src
             else:
                 raise ValueError(f'Invalid constant assignment: {field}')
     if assign_formats:
-        dict_formats = OrderedDict()
         fields = assign_formats.split(',')
         for field in fields:
             if '=' in field:
                 dst, src = field.split('=')
-                dict_formats[dst] = src
+                config.process.assign_formats[dst] = src
             else:
                 raise ValueError(f'Invalid template assignment: {field}')
     if pickup_columns:
@@ -286,20 +243,14 @@ def convert(
             else:
                 raise ValueError(f'Invalid split by newline: {field}')
     if fields_to_assign_ids:
-        dict_assign_ids = OrderedDict()
-        fields = fields_to_assign_ids.split(',')
-        for field in fields:
-            if '=' in field:
-                dst, src = field.split('=')
-                dict_assign_ids[dst] = src
-            else:
-                raise ValueError(f'Invalid id assignment: {field}')
+        setup_assign_ids(config, fields_to_assign_ids)
     if output_file:
         ext = os.path.splitext(output_file)[1]
         if ext not in dict_savers:
             raise ValueError(f'Unsupported file type: {ext}')
         saver = dict_savers[ext]
     ic(config)
+    #return # debug return
     for input_file in input_files:
         ic(input_file)
         if not os.path.exists(input_file):
@@ -319,22 +270,23 @@ def convert(
         for index, row in df.iterrows():
             orig = OrderedDict(row)
             new_row = OrderedDict(row)
-            set_field_value(new_row, '__debug__.__original__', orig)
-            set_field_value(new_row, '__debug__.__file__', input_file)
-            if dict_constants:
-                new_row = map_constants(new_row, dict_constants)
+            if STAGING_FIELD not in new_row:
+                set_field_value(new_row, f'{STAGING_FIELD}.{INPUT_FIELD}', orig)
+                set_field_value(new_row, f'{STAGING_FIELD}.{FILE_FIELD}', input_file)
+            if config.process.assign_constants:
+                new_row = map_constants(new_row, config.process.assign_constants)
             if config.map:
                 new_row = remap_columns(new_row, config.map)
             if config.process.split_by_newline:
                 new_row = apply_fields_split_by_newline(new_row, config.process.split_by_newline)
-            if dict_assign_ids:
-                new_row = assign_id(new_row, dict_assign_ids, root_id_stat)
-            if dict_formats:
-                new_row = map_formats(new_row, dict_formats)
+            if config.process.assign_ids:
+                new_row = assign_id(new_row, config.process.assign_ids, id_context_map)
+            if config.process.assign_formats:
+                new_row = map_formats(new_row, config.process.assign_formats)
             if config.map:
                 new_row = remap_columns(new_row, config.map)
             if not output_debug:
-                new_row.pop('__debug__', None)
+                new_row.pop(STAGING_FIELD, None)
             new_rows.append(new_row)
         new_df = pd.DataFrame(new_rows)
         df_list.append(new_df)
